@@ -154,7 +154,7 @@ static int IsValidCipherSuite(const char* line, char *suite, size_t suite_spc)
                 printf("suite too long!\n");
                 return 0;
             }
-            XMEMCPY(suite, begin, (size_t) len);
+            XMEMCPY(suite, begin, len);
             suite[len] = '\0';
         }
         else
@@ -660,7 +660,7 @@ static void test_harness(void* vargs)
         return;
     }
 
-    script = (char*)malloc((size_t)(sz+1));
+    script = (char*)malloc(sz+1);
     if (script == 0) {
         fprintf(stderr, "unable to allocate script buffer\n");
         fclose(file);
@@ -668,7 +668,7 @@ static void test_harness(void* vargs)
         return;
     }
 
-    len = (long) fread(script, 1, (size_t)sz, file);
+    len = fread(script, 1, sz, file);
     if (len != sz) {
         fprintf(stderr, "read error\n");
         fclose(file);
@@ -1455,8 +1455,8 @@ exit:
 
     return args.return_code;
 #else
+    return NOT_COMPILED_IN;
     (void)argc;
     (void)argv;
-    return NOT_COMPILED_IN;
 #endif /* !NO_WOLFSSL_SERVER && !NO_WOLFSSL_CLIENT */
 }
