@@ -59,7 +59,7 @@ struct alignas(CPU_LEVEL1_DCACHE_LINESIZE) trx_rseg_t
   /** tablespace containing the rollback segment; constant after init() */
   fil_space_t *space;
   /** latch protecting everything except page_no, space */
-  IF_DBUG(srw_lock_debug,srw_spin_lock) latch;
+  srw_spin_lock latch;
   /** rollback segment header page number; constant after init() */
   uint32_t page_no;
   /** length of the TRX_RSEG_HISTORY list (number of transactions) */
@@ -170,21 +170,19 @@ public:
   /** Last not yet purged undo log header; FIL_NULL if all purged */
   uint32_t last_page_no;
 
-  /** trx_t::no << 16 | last_offset */
+  /** trx_t::no | last_offset << 48 */
   uint64_t last_commit_and_offset;
 
   /** @return the commit ID of the last committed transaction */
   trx_id_t last_trx_no() const
-  { return last_commit_and_offset >> 16; }
+  { return last_commit_and_offset & ((1ULL << 48) - 1); }
   /** @return header offset of the last committed transaction */
   uint16_t last_offset() const
-  {
-    return static_cast<uint16_t>(last_commit_and_offset);
-  }
+  { return static_cast<uint16_t>(last_commit_and_offset >> 48); }
 
   void set_last_commit(uint16_t last_offset, trx_id_t trx_no)
   {
-    last_commit_and_offset= trx_no << 16 | static_cast<uint64_t>(last_offset);
+    last_commit_and_offset= static_cast<uint64_t>(last_offset) << 48 | trx_no;
   }
 
   /** @return the page identifier */
@@ -295,7 +293,10 @@ which corresponds to the transaction just being committed.
 In a replication slave, this updates the master binlog position
 up to which replication has proceeded.
 @param[in,out]	rseg_header	rollback segment header
-@param[in]	trx		committing transaction
+@param[in]	log_file_name	binlog file name
+@param[in]	log_offset	binlog offset value
 @param[in,out]	mtr		mini-transaction */
-void trx_rseg_update_binlog_offset(buf_block_t *rseg_header, const trx_t *trx,
+void trx_rseg_update_binlog_offset(buf_block_t *rseg_header,
+                                   const char *log_file_name,
+                                   ulonglong log_offset,
                                    mtr_t *mtr);
